@@ -218,7 +218,7 @@ LIBUSB_CLASS_IMAGE = LIBUSB_CLASS_PTP
 
 # Descriptor types as defined by the USB specification.
 libusb_descriptor_type = Enum({
-# Device descriptor. See libusb_device_descriptor. 
+# Device descriptor. See libusb_device_descriptor.
 'LIBUSB_DT_DEVICE': 0x01,
 # Configuration descriptor. See libusb_config_descriptor.
 'LIBUSB_DT_CONFIG': 0x02,
@@ -522,6 +522,17 @@ libusb_speed = Enum({
 'LIBUSB_SPEED_SUPER': 4,
 })
 
+libusb_supported_speed = Enum({
+# Low speed operation supported (1.5MBit/s).
+'LIBUSB_LOW_SPEED_OPERATION': 1,
+# Full speed operation supported (12MBit/s).
+'LIBUSB_FULL_SPEED_OPERATION': 2,
+# High speed operation supported (480MBit/s).
+'LIBUSB_HIGH_SPEED_OPERATION': 4,
+# Superspeed operation supported (5000MBit/s).
+'LIBUSB_5GBPS_OPERATION': 8,
+})
+
 # Error codes. Most libusb functions return 0 on success or one of these
 # codes on failure.
 libusb_error = Enum({
@@ -578,17 +589,17 @@ libusb_transfer_status = Enum({
 # libusb_transfer.flags values
 libusb_transfer_flags = Enum({
 # Report short frames as errors
-'LIBUSB_TRANSFER_SHORT_NOT_OK': 1<<0,
+'LIBUSB_TRANSFER_SHORT_NOT_OK': 1 << 0,
 # Automatically free() transfer buffer during libusb_free_transfer()
-'LIBUSB_TRANSFER_FREE_BUFFER': 1<<1,
+'LIBUSB_TRANSFER_FREE_BUFFER': 1 << 1,
 # Automatically call libusb_free_transfer() after callback returns.
 # If this flag is set, it is illegal to call libusb_free_transfer()
 # from your transfer callback, as this will result in a double-free
 # when this flag is acted upon.
-'LIBUSB_TRANSFER_FREE_TRANSFER': 1<<2,
+'LIBUSB_TRANSFER_FREE_TRANSFER': 1 << 2,
 # Terminate transfers that are a multiple of the endpoint's
 # wMaxPacketSize with an extra zero length packet.
-'LIBUSB_TRANSFER_ADD_ZERO_PACKET': 1<<3,
+'LIBUSB_TRANSFER_ADD_ZERO_PACKET': 1 << 3,
 })
 
 # Isochronous packet descriptor.
@@ -633,7 +644,21 @@ libusb_transfer._fields_ = _libusb_transfer_fields
 
 libusb_capability = Enum({
 # The libusb_has_capability() API is available.
-'LIBUSB_CAP_HAS_CAPABILITY': 0,
+'LIBUSB_CAP_HAS_CAPABILITY': 0x0000,
+# Hotplug support is available.
+'LIBUSB_CAP_HAS_HOTPLUG': 0x0001,
+# The library can access HID devices without requiring user intervention.
+'LIBUSB_CAP_HAS_HID_ACCESS': 0x0100,
+# The library supports detaching of the default USB driver.
+'LIBUSB_CAP_SUPPORTS_DETACH_KERNEL_DRIVER': 0x0101,
+})
+
+libusb_log_level = Enum({
+'LIBUSB_LOG_LEVEL_NONE': 0,
+'LIBUSB_LOG_LEVEL_ERROR': 1,
+'LIBUSB_LOG_LEVEL_WARNING': 2,
+'LIBUSB_LOG_LEVEL_INFO': 3,
+'LIBUSB_LOG_LEVEL_DEBUG': 4,
 })
 
 #int libusb_init(libusb_context **ctx);
@@ -678,16 +703,16 @@ except AttributeError:
 else:
     libusb_error_name.argtypes = [c_int]
     libusb_error_name.restype = c_char_p
-try:
-    #char *libusb_strerror(enum libusb_error errcode);
-    libusb_strerror = libusb.libusb_strerror
-except AttributeError:
-    # Place holder
-    def libusb_strerror(errcode):
-        return None
-else:
-    libusb_strerror.argtypes = [c_int]
-    libusb_strerror.restype = c_char_p
+
+# Note on libusb_strerror, libusb_setlocale and future functions in the
+# same spirit:
+# I do not think end-user-facing messages belong to a technical library.
+# Such features bring a new, non essential set of problems, and is a luxury
+# I do not want to spend time supporting considering limited resources and
+# more important stuff to work on.
+# For backward compatibility, expose libusb_strerror placeholder.
+def libusb_strerror(errcode):
+    return None
 
 #ssize_t libusb_get_device_list(libusb_context *ctx,
 #        libusb_device ***list);
@@ -739,6 +764,33 @@ libusb_free_config_descriptor.restype = None
 libusb_get_bus_number = libusb.libusb_get_bus_number
 libusb_get_bus_number.argtypes = [libusb_device_p]
 libusb_get_bus_number.restype = c_uint8
+try:
+    #uint8_t libusb_get_port_number(libusb_device *dev);
+    libusb_get_port_number = libusb.libusb_get_port_number
+except AttributeError:
+    pass
+else:
+    libusb_get_port_number.argtypes = [libusb_device_p]
+    libusb_get_port_number.restype = c_uint8
+try:
+    #int libusb_get_port_numbers(libusb_device *dev,
+    #       uint8_t* port_numbers, int port_numbers_len);
+    libusb_get_port_numbers = libusb.libusb_get_port_numbers
+except AttributeError:
+    pass
+else:
+    libusb_get_port_numbers.argtypes = [libusb_device_p, POINTER(c_uint8),
+        c_int]
+    libusb_get_port_numbers.restype = c_int
+# Missing: libusb_get_port_path (deprecated since 1.0.16)
+try:
+    #libusb_device * LIBUSB_CALL libusb_get_parent(libusb_device *dev);
+    libusb_get_parent = libusb.libusb_get_parent
+except AttributeError:
+    pass
+else:
+    libusb_get_parent.argtypes = [libusb_device_p]
+    libusb_get_parent.restype = libusb_device_p
 #uint8_t libusb_get_device_address(libusb_device *dev);
 libusb_get_device_address = libusb.libusb_get_device_address
 libusb_get_device_address.argtypes = [libusb_device_p]
@@ -817,6 +869,17 @@ libusb_detach_kernel_driver.argtypes = [libusb_device_handle_p, c_int]
 #int libusb_attach_kernel_driver(libusb_device_handle *dev, int interface);
 libusb_attach_kernel_driver = libusb.libusb_attach_kernel_driver
 libusb_attach_kernel_driver.argtypes = [libusb_device_handle_p, c_int]
+try:
+    #int libusb_set_auto_detach_kernel_driver(
+    #       libusb_device_handle *dev, int enable);
+    libusb_set_auto_detach_kernel_driver = \
+        libusb.libusb_set_auto_detach_kernel_driver
+except AttributeError:
+    pass
+else:
+    libusb_set_auto_detach_kernel_driver.argtypes = [libusb_device_handle_p,
+        c_int]
+    libusb_set_auto_detach_kernel_driver.restype = c_int
 
 # Get the data section of a control transfer. This convenience function is here
 # to remind you that the data does not start until 8 bytes into the actual
@@ -983,7 +1046,7 @@ def libusb_get_iso_packet_buffer_simple(transfer_p, packet):
         return None
     iso_length = transfer.iso_packet_desc.length
     return _get_iso_packet_buffer(transfer, iso_length * packet, iso_length)
- 
+
 # sync I/O
 
 #int libusb_control_transfer(libusb_device_handle *dev_handle,
@@ -1110,5 +1173,51 @@ libusb_set_pollfd_notifiers.argtypes = [libusb_context_p,
                                         libusb_pollfd_removed_cb_p, py_object]
 libusb_set_pollfd_notifiers.restype = None
 
-# /libusb.h
+#typedef int libusb_hotplug_callback_handle;
+libusb_hotplug_callback_handle = c_int
 
+libusb_hotplug_flag = Enum({
+'LIBUSB_HOTPLUG_ENUMERATE': 1,
+})
+
+libusb_hotplug_event = Enum({
+'LIBUSB_HOTPLUG_EVENT_DEVICE_ARRIVED': 0x01,
+'LIBUSB_HOTPLUG_EVENT_DEVICE_LEFT': 0x02,
+})
+
+LIBUSB_HOTPLUG_MATCH_ANY = -1
+
+#typedef int (*libusb_hotplug_callback_fn)(libusb_context *ctx,
+#        libusb_device *device, libusb_hotplug_event event, void *user_data);
+libusb_hotplug_callback_fn_p = CFUNCTYPE(c_int, libusb_context_p,
+    libusb_device_p, c_int, c_void_p)
+
+#int libusb_hotplug_register_callback(libusb_context *ctx,
+#        libusb_hotplug_event events, libusb_hotplug_flag flags,
+#        int vendor_id, int product_id, int dev_class,
+#        libusb_hotplug_callback_fn cb_fn, void *user_data,
+#        libusb_hotplug_callback_handle *handle);
+try:
+    libusb_hotplug_register_callback = libusb.libusb_hotplug_register_callback
+except AttributeError:
+    pass
+else:
+    libusb_hotplug_register_callback.argtypes = [libusb_context_p,
+        c_int, c_int,
+        c_int, c_int, c_int,
+        libusb_hotplug_callback_fn_p, c_void_p,
+        POINTER(libusb_hotplug_callback_handle)]
+    libusb_hotplug_register_callback.restype = c_int
+
+#void libusb_hotplug_deregister_callback(libusb_context *ctx,
+#        libusb_hotplug_callback_handle handle);
+try:
+    libusb_hotplug_deregister_callback = libusb.libusb_hotplug_deregister_callback
+except AttributeError:
+    pass
+else:
+    libusb_hotplug_deregister_callback.argtypes = [libusb_context_p,
+        libusb_hotplug_callback_handle]
+    libusb_hotplug_deregister_callback.restype = None
+
+# /libusb.h
