@@ -21,14 +21,13 @@ Features:
 
 import libusb1
 from ctypes import byref, create_string_buffer, c_int, sizeof, POINTER, \
-    cast, c_uint16, c_ubyte, string_at, c_void_p, cdll
+    cast, c_uint8, c_uint16, c_ubyte, string_at, c_void_p, cdll, addressof
 import sys
 import threading
 from ctypes.util import find_library
 import warnings
 import weakref
 import collections
-#import select
 try:
     namedtuple = collections.namedtuple
 except AttributeError:
@@ -88,6 +87,10 @@ except AttributeError:
 # From a comment in libusb-1.0: "Some devices choke on size > 255"
 STRING_LENGTH = 255
 
+# As of v3 of USB specs, there cannot be more than 7 hubs from controler to
+# device.
+PATH_MAX_DEPTH = 7
+
 EVENT_CALLBACK_SET = frozenset((
     libusb1.LIBUSB_TRANSFER_COMPLETED,
     libusb1.LIBUSB_TRANSFER_ERROR,
@@ -262,7 +265,7 @@ class USBTransfer(object):
             raise DoomedTransferError('Cannot reuse a doomed transfer')
         if isinstance(buffer_or_len, (int, long)):
             length = buffer_or_len
-            string_buffer = create_binary_buffer(length + \
+            string_buffer = create_binary_buffer(length +
                 libusb1.LIBUSB_CONTROL_SETUP_SIZE)
         else:
             length = len(buffer_or_len)
@@ -338,7 +341,7 @@ class USBTransfer(object):
         self.__transfer_buffer = string_buffer
         self.__user_data = user_data
         libusb1.libusb_fill_interrupt_transfer(self.__transfer, self.__handle,
-            endpoint, string_buffer,  sizeof(string_buffer),
+            endpoint, string_buffer, sizeof(string_buffer),
             self.__ctypesCallbackWrapper, None, timeout)
         self.__callback = callback
         self.__initialized = True
@@ -744,37 +747,6 @@ class USBPollerThread(threading.Thread):
         self.__fd_set.discard(fd)
         self.__poller.unregister(fd)
 
-#class KQueuePoller(object):
-#    """
-#    Minmal kqueue wrapper to expose poll-ish API.
-#    """
-#    def __init__(self, kqueue):
-#        self._kqueue = kqueue
-#        self._fd_set = set()
-#
-#    def register(self, fd, events):
-#        kevent = select.kevent(fd, flags=select.KQ_EV_ADD)
-#        # It's needed to duplicate poll event values, so this works when they
-#        # are not available from select module.
-#        if events & :
-#            kevent.filter = select.KQ_FILTER_READ
-#            self._kqueue.control()
-#        if events & :
-#            kevent.filter = select.KQ_FILTER_WRITE
-#        self._kqueue.control([select.kevent(fd, kq_events, select.KQ_EV_ADD)], 0)
-#        # fd come from libusb, they are file descriptors, not python objects
-#        assert isinstance(fd, int), fd
-#        self._fd_set.add(fd)
-#
-#    def unregister(self, fd):
-#        self._kqueue.control([select.kevent(fd, 0, select.KQ_EV_DELETE)], 0)
-#        # fd come from libusb, they are file descriptors, not python objects
-#        assert isinstance(fd, int), fd
-#        self._fd_set.remove(fd)
-#
-#    def poll(self, timeout=None):
-#        self._kqueue.control(None, len(self._fd_set), timeout * 1000.)
-
 class USBPoller(object):
     """
     Class allowing integration of USB event polling in a file-descriptor
@@ -1048,6 +1020,17 @@ class USBDeviceHandle(object):
         if result:
             raise libusb1.USBError(result)
 
+    def setAutoDetachKernelDriver(self, enable):
+        """
+        Control automatic kernel driver detach.
+        enable (bool)
+            True to enable auto-detach, False to disable it.
+        """
+        result = libusb1.libusb_set_auto_detach_kernel_driver(self.__handle,
+            bool(enable))
+        if result:
+            libusb1.USBError(result)
+
     def getSupportedLanguageList(self):
         """
         Return a list of USB language identifiers (as integers) supported by
@@ -1254,6 +1237,8 @@ class USBConfiguration(object):
     def getNumInterfaces(self):
         return self.__config.bNumInterfaces
 
+    __len__ = getNumInterfaces
+
     def getConfigurationValue(self):
         return self.__config.bConfigurationValue
 
@@ -1312,6 +1297,8 @@ class USBInterface(object):
     def getNumSettings(self):
         return self.__interface.num_altsetting
 
+    __len__ = getNumSettings
+
     def __iter__(self):
         """
         Iterates over settings in this insterface, yielding
@@ -1353,6 +1340,8 @@ class USBInterfaceSetting(object):
     def getNumEndpoints(self):
         return self.__alt_setting.bNumEndpoints
 
+    __len__ = getNumEndpoints
+
     def getClass(self):
         return self.__alt_setting.bInterfaceClass
 
@@ -1435,15 +1424,13 @@ class USBDevice(object):
     __libusb_free_config_descriptor = libusb1.libusb_free_config_descriptor
     __byref = byref
 
-    def __init__(self, context, device_p):
+    def __init__(self, context, device_p, can_load_configuration=True):
         """
         You should not instanciate this class directly.
         Call LibUSBContext methods to receive instances of this class.
         """
-        # Important: device_p refcount must be incremented before being given
-        # to this constructor. This class will decrement refcount upon
-        # destruction.
         self.__context = context
+        libusb1.libusb_ref_device(device_p)
         self.device_p = device_p
         # Fetch device descriptor
         device_descriptor = libusb1.libusb_device_descriptor()
@@ -1452,20 +1439,23 @@ class USBDevice(object):
         if result:
             raise libusb1.USBError(result)
         self.device_descriptor = device_descriptor
-        # Fetch all configuration descriptors
-        self.__configuration_descriptor_list = []
-        append = self.__configuration_descriptor_list.append
-        for configuration_id in xrange(device_descriptor.bNumConfigurations):
-            config = libusb1.libusb_config_descriptor_p()
-            result = libusb1.libusb_get_config_descriptor(device_p,
-                configuration_id, byref(config))
-            if result == libusb1.LIBUSB_ERROR_NOT_FOUND:
-                # Some devices (ex windows' root hubs) tell they have one
-                # configuration, but they have no configuration descriptor.
-                continue
-            if result:
-                raise libusb1.USBError(result)
-            append(config.contents)
+        if can_load_configuration:
+            self.__configuration_descriptor_list = descriptor_list = []
+            append = descriptor_list.append
+            device_p = self.device_p
+            for configuration_id in xrange(
+                    self.device_descriptor.bNumConfigurations):
+                config = libusb1.libusb_config_descriptor_p()
+                result = libusb1.libusb_get_config_descriptor(device_p,
+                    configuration_id, byref(config))
+                if result == libusb1.LIBUSB_ERROR_NOT_FOUND:
+                    # Some devices (ex windows' root hubs) tell they have
+                    # one configuration, but they have no configuration
+                    # descriptor.
+                    continue
+                if result:
+                    raise libusb1.USBError(result)
+                append(config.contents)
 
     def __del__(self):
         self.__libusb_unref_device(self.device_p)
@@ -1481,6 +1471,9 @@ class USBDevice(object):
             self.getProductID(),
         )
 
+    def __len__(self):
+        return len(self.__configuration_descriptor_list)
+
     def __getitem__(self, index):
         return USBConfiguration(self.__configuration_descriptor_list[index])
 
@@ -1492,8 +1485,8 @@ class USBDevice(object):
     iterConfiguations = iterConfigurations
 
     def iterSettings(self):
-        for config in self.__configuration_descriptor_list:
-            for interface in USBConfiguration(config):
+        for config in self.iterConfigurations():
+            for interface in config:
                 for setting in interface:
                     yield setting
 
@@ -1503,6 +1496,26 @@ class USBDevice(object):
         """
         return libusb1.libusb_get_bus_number(self.device_p)
 
+    def getPortNumber(self):
+        """
+        Get device's port number.
+        """
+        return libusb1.libusb_get_port_number(self.device_p)
+
+    def getPortNumberList(self):
+        """
+        Get the port number of each hub toward device.
+        """
+        port_list = (c_uint8 * PATH_MAX_DEPTH)()
+        result = libusb1.libusb_get_port_numbers(self.device_p, port_list,
+            len(port_list))
+        if result < 0:
+            raise libusb1.USBError(result)
+        return list(port_list[:result])
+
+    # TODO: wrap libusb_get_parent when/if libusb removes the need to be inside
+    # a libusb_(get|free)_device_list block.
+
     def getDeviceAddress(self):
         """
         Get device's address on its bus.
@@ -1700,6 +1713,7 @@ class USBContext(object):
         if result:
             raise libusb1.USBError(result)
         self.__context_p = context_p
+        self.__hotplug_callback_dict = {}
 
     def __del__(self):
         # Avoid locking.
@@ -1757,10 +1771,10 @@ class USBContext(object):
             for device_p in device_p_p[:device_list_len]:
                 try:
                     # Instanciate our own libusb_device_p object so we can free
-                    # libusb-provided device list. Is this a bug in ctypes that it
-                    # doesn't copy pointer value (=pointed memory address) ? At
-                    # least, it's not so convenient and forces using such weird
-                    # code.
+                    # libusb-provided device list. Is this a bug in ctypes that
+                    # it doesn't copy pointer value (=pointed memory address) ?
+                    # At least, it's not so convenient and forces using such
+                    # weird code.
                     device = USBDevice(self, libusb_device_p(device_p.contents))
                 except libusb1.USBError:
                     if not skip_on_error:
@@ -1768,7 +1782,7 @@ class USBContext(object):
                 else:
                     append(device)
         finally:
-            libusb1.libusb_free_device_list(device_p_p, 0)
+            libusb1.libusb_free_device_list(device_p_p, 1)
         return result
 
     def getByVendorIDAndProductID(self, vendor_id, product_id,
@@ -1995,6 +2009,60 @@ class USBContext(object):
         """
         return libusb1.libusb_has_capability(capability)
 
+    @_validContext
+    def hotplugRegisterCallback(self, callback,
+                events=libusb1.LIBUSB_HOTPLUG_EVENT_DEVICE_ARRIVED | \
+                    libusb1.LIBUSB_HOTPLUG_EVENT_DEVICE_LEFT,
+                flags=libusb1.LIBUSB_HOTPLUG_ENUMERATE,
+                vendor_id=libusb1.LIBUSB_HOTPLUG_MATCH_ANY,
+                product_id=libusb1.LIBUSB_HOTPLUG_MATCH_ANY,
+                dev_class=libusb1.LIBUSB_HOTPLUG_MATCH_ANY,
+            ):
+        """
+        Registers an hotplug callback.
+        On success, returns an opaque value which can be passed to
+        hotplugDeregisterCallback.
+        Callback must accept the following positional arguments:
+        - this USBContext instance
+        - an USBDevice instance
+          If device has left, configuration descriptors may not be
+          available. Its device descriptor will be available.
+        - event type (see libusb1.libusb_hotplug_event)
+        Callback must return whether it must be unregistered (any true value
+        to be unregistered, any false value to be kept registered).
+        """
+        def wrapped_callback(context_p, device_p, event, _):
+            assert addressof(context_p.contents) == addressof(
+                self.__context_p.contents), (context_p, self.__context_p)
+            unregister = bool(callback(self, USBDevice(self, device_p,
+                event != libusb1.LIBUSB_HOTPLUG_EVENT_DEVICE_LEFT), event))
+            if unregister:
+                del self.__hotplug_callback_dict[handle]
+            return unregister
+        handle = c_int()
+        callback_p = libusb1.libusb_hotplug_callback_fn_p(wrapped_callback)
+        result = libusb1.libusb_hotplug_register_callback(self.__context_p,
+            events, flags, vendor_id, product_id, dev_class, callback_p,
+            None, byref(handle))
+        if result:
+            raise libusb1.USBError(result)
+        handle = handle.value
+        # Keep strong references
+        assert handle not in self.__hotplug_callback_dict, (handle,
+            self.__hotplug_callback_dict)
+        self.__hotplug_callback_dict[handle] = (callback_p, wrapped_callback)
+        return handle
+
+    @_validContext
+    def hotplugDeregisterCallback(self, handle):
+        """
+        Deregisters an hotplug callback.
+        handle (opaque)
+            Return value of a former hotplugRegisterCallback call.
+        """
+        del self.__hotplug_callback_dict[handle]
+        libusb1.libusb_hotplug_deregister_callback(self.__context_p, handle)
+
 del USBContext._validContext
 
 def getVersion():
@@ -2021,4 +2089,3 @@ class LibUSBContext(USBContext):
         warnings.warn('LibUSBContext is being renamed to USBContext',
             DeprecationWarning)
         super(LibUSBContext, self).__init__()
-
